### USER-DEFINED SECTION; edit these values ###
source("Get-noncentralities-from-table.r")
# comment out previous line and
# uncomment next line if want to fit effect size distribution parameters from GWAS p-values
# source("Estimate-effects-distribution.r")
k = 1e6 # total number of tests, "L+m"
u = 200 # compute rFDR for this value of top hits
r.FDR = 0.5 # compute number of top hits for this value of rFDR
max.u = round(0.4 * k) # top hits examined should not exceed this fraction of k
### END OF USER-DEFINED SECTION ###

Params <- c(qShape, qScale, At) # (shape, scale, truncation)
m <- sum(Nloc)

# right-truncated gamma density
fd.dist <- function(x, Params) {
  dgamma(x, shape=Params[1], scale=Params[2]) /
    pgamma(Params[3], shape=Params[1], scale=Params[2])
}

fy.int <- function(nc, ar, Params) {
  fd.dist(nc, Params) * dchisq(ar, df=1, ncp=nc)
}

# p-value density for true effects
fy.marg <- function(pval, Params) {
  ar <- qchisq(pval, df=1, lower.tail=FALSE)
  marg <- integrate(fy.int, 0, Params[3], ar,
     Params)$value / dchisq(ar, df=1)
  return (marg)
}

Fy <- function(y, nc, df=1) { 1-pchisq(qchisq(1-y,df=df),df=df,ncp=nc) }

Fy.int <- function(nc, pval, Params) {
  fd.dist(nc, Params) * Fy(pval,nc)
}

# marginal p-value CDF for true effects
Fy.marg <- function(pval, Params) {
  marg <- integrate(Fy.int, 0, Params[3], pval,
     Params)$value 
  return (marg)
}

# j-th ordered marginal p-value CDF for true effects
Fyj.marg <- function(y, j, m, Params) {
  1 - pbinom(j-1, m, Fy.marg(y, Params))
}

# posterior Pr(H_0 | p-value)
post0.marg <- function(pval, p0, Params) {
  marg <- fy.marg(pval, Params)
  (1 / (1 + (1-p0)/p0 * marg))
}

get.Fdr <- function(v,m,Params) {
  s <- 0
  for(j in 1:min(v,m)) {
    s <- s + Fyj.marg((v-j+1)/((k-m+1)*(1+1/v)), j, m, Params)
  }
  rFDR <- 1 - s/v
}

if(u>1) {
  rFDR <- get.Fdr(u,m,Params);
  cat("Proportion of false positives among top", u, "p-values =", rFDR, "\n")
}

diff.Fdr <- function(v,fdr,m,Params) {
  s <- 0
  for(j in 1:min(v,m)) {
    s <- s + Fyj.marg((v-j+1)/((k-m+1)*(1+1/v)), j, m, Params)
  }
  rFDR <- 1 - s/v
  abs(rFDR - fdr)
}

if(r.FDR > 0 && r.FDR < 1) {
  uu <- optimize(diff.Fdr, interval=c(1, max.u),
          tol=0.01, fdr=r.FDR, m=m, Params=Params)$minimum
  uu =  round(uu)
  cat("\nNumber of top p-values for rFDR to be <=", r.FDR, "is", uu, "\n")
  cat("rFDR at that number =", get.Fdr(uu,m,Params), "\n")
}
