### USER-DEFINED SECTION; edit these values ###
source("Get-noncentralities-from-table.r")
k = 1e6 # number of tests
# generate correlated p-values
# note: these parameters are tuned to get the extremal index = 0.4
# for k = 1e6; the extremal index value will change if different
# k value is assumed
dt=0.077; mb=0.5; s2=100
### END OF USER-DEFINED SECTION

m <- sum(Nloc)  # number of true effects
L = k-m # number of false effects

# random numbers from right-truncated gamma density
frnd.dist <- function(n, Prior) {
    x <- runif(n)
    xx <- x*pgamma(Prior[3], shape=Prior[1], scale=Prior[2]) # Prior[3]: truncated value
    qgamma(xx, shape=Prior[1], scale=Prior[2])
}

# random p-values for true effects
Rpns <- function(n, Prior) {
  repeat {
    ncent <- frnd.dist(n, Prior) # sampling random noncentralities
    z <- rnorm(n, mean=sqrt(ncent), sd=1)
    pv <- pchisq(z^2, df=1, lower.tail = FALSE)
    if(min(pv) > 0) return (pv)
  }
}

# ou & Gy: generating correlated p-values from Ornstein-Uhlenbeck diffusion
ou <- function(x, dt, mbar, s2, xnew, fa) {
    vr <- s2*(1 - exp(-2*mbar*dt))/(2*mbar)
    x <- x*exp(-mbar*dt) + fa/mbar*(1-exp(-mbar*dt)) + sqrt(vr)*xnew
}

Gy <- function(dt,mb,s2,n,fa=0) {
    err <- rnorm(n)
    y <- array(0,n); y[1] <- err[1]
    for(i in 2:length(y)) y[i] <- ou(y[i-1], dt, mb, s2, err[i], fa)
    return (y)
}

OneTail <- function(y,mb,s2,fa=0) { # 1-tailed p
  p <- pnorm(y, fa, sqrt(s2/(2*mb)), lower.tail = FALSE)
}

pTrue <- Rpns(m, c(qShape, qScale, At))
fa=0
pFalse <- OneTail(Gy(dt=dt, mb=mb, s2=s2, n=L, fa=fa), mb=mb, s2=s2, fa=fa)
pFalse <- 2 * pmin(pFalse, 1-pFalse)
ptr <- c(pTrue, pFalse)
ind <- c(rep("TA",m), rep("FA",L))
d <- data.frame(ptr, ind)
names(d) = c("Pvalue", "SNP")
top.x <- d[order(d$Pvalue),][1:200,]
write.csv(format(top.x, digits=3), file="pvalues.csv", row.names=FALSE)
gzipped.all <- gzfile("pvalues-all.csv.gz", "w")
write.csv(format(d, digits=3), file=gzipped.all, row.names=FALSE)
close(gzipped.all)
